@@ -2,79 +2,6 @@ use proc_macro::{self, TokenStream};
 use quote::{quote, format_ident};
 use syn::{parse_macro_input, DeriveInput, FieldsNamed};
 
-/// # field_accessor
-/// 
-/// With this procedural macro, you can dynamically get and update a field of a struct by a `String` type variable.
-/// This program is currently experimental.
-/// This can be useful if you don't know which field you want when compiling.
-/// ## Installation
-/// 
-/// ```ignore
-/// [dependencies]
-/// field_accessor = "0"
-/// ```
-/// 
-/// ## About this macro
-/// This macro provides the two methods for structs by implementing `GetterSetter` trait. Using `get` you can get a field's value dynamically.
-/// Also, a field's value can be updated by `set`. The functionality is similar to python's `getattr`, `setattr`.
-/// ```rust
-/// trait GetterSetter<T> {
-///     fn get(&mut self, field_string: &String) -> Result<&T, String>;
-///     fn set(&mut self, field_string: &String, value: T) -> Result<(), String>;
-/// }
-/// ```
-/// 
-/// ## Usage and Example
-/// ```rust
-/// use field_accessor::FieldAccessor;
-/// 
-/// #[derive(FieldAccessor)]
-/// struct Dog {
-///     name: String,
-///     age: u32,
-///     life_expectancy: u32,
-/// }
-/// 
-/// fn main() {
-///     let mut dog = Dog {
-///         name: "Taro".to_string(),
-///         age: 3,
-///         life_expectancy: 9,
-///     };
-/// 
-///     let field_name = "name".to_string();
-///     let value_to_update = "Jiro".to_string();
-///     dog.set(&field_name, value_to_update).unwrap();
-///     let value_on_error;
-///     let fieldvalue: &String = match dog.get(&"invalid_field".to_string()) {
-///         Ok(value) => value,
-///         Err(_) => {value_on_error = "Ken".to_string(); &value_on_error},
-///     };
-///     println!("{:?}", fieldvalue);
-/// 
-///     let field_name = "age".to_string();
-///     let value_to_update = 4u32;
-///     dog.set(&field_name, value_to_update).unwrap();
-///     let fieldvalue: &u32 = dog.get(&field_name).unwrap();
-///     println!("{:?}", fieldvalue);
-/// 
-///     let field_name = "life_expectancy".to_string();
-///     let value_to_update = 10u32;
-///     dog.set(&field_name, value_to_update).unwrap();
-///     let fieldvalue: &u32 = dog.get(&field_name).unwrap();
-///     println!("{:?}", fieldvalue);
-/// 
-/// }
-/// 
-/// ```
-/// ### output
-/// ```ignore
-/// "Ken"
-/// 4
-/// 10
-/// ```
-/// 
-/// This code is generated at compiling.
 #[proc_macro_derive(FieldAccessor)]
 pub fn derive(input: TokenStream) -> TokenStream {
     let DeriveInput { ident, data, .. } = parse_macro_input!(input);
@@ -89,25 +16,54 @@ pub fn derive(input: TokenStream) -> TokenStream {
                 let enumname = format_ident!("{}{}", ident, "FieldEnum");
                 let structinfo = format_ident!("{}{}", ident, "StructInfo");
 
-                let mut set_quotes = vec![];
                 let mut get_quotes = vec![];
-                let mut set_tys = vec![];
+                let mut get_mut_quotes = vec![];
+                let mut take_quotes = vec![];
+                let mut replace_quotes = vec![];
+                let mut set_quotes = vec![];
                 let mut get_tys = vec![];
+                let mut get_mut_tys = vec![];
+                let mut take_tys = vec![];
+                let mut replace_tys = vec![];
+                let mut swap_tys = vec![];
+                let mut set_tys = vec![];
 
                 let field_idents = named
                     .iter()
                     .map(|f| &f.ident);
 
+                let mut swap_ident = vec![];
+                let mut swap_ident2 = vec![];
+                for (outer_ident, outer_type) in named.iter().map(|f| &f.ident).zip(named.iter().map(|f| &f.ty)){
+                    for (inner_ident, inner_type) in named.iter().map(|f| &f.ident).zip(named.iter().map(|f| &f.ty)){
+                        if outer_type == inner_type {
+                            if outer_ident != inner_ident{
+                                swap_tys.push(inner_type);
+                                swap_ident.push(outer_ident.clone());
+                                swap_ident2.push(inner_ident.clone());
+                            }
+                        }
+                    }
+                }
+
                 for name in named.clone().iter() {
                     if get_tys.contains(&name.ty) {
                     } else {
                         get_tys.push(name.ty.clone());
+                        get_mut_tys.push(name.ty.clone());
+                        take_tys.push(name.ty.clone());
+                        replace_tys.push(name.ty.clone());
                         set_tys.push(name.ty.clone());
+                        
                         let get_filtered_ident = named
                             .iter()
                             .filter(|x| x.ty == name.ty)
                             .map(|f| &f.ident);
+                        let get_mut_filtered_ident = get_filtered_ident.clone();
+                        let take_filtered_ident = get_filtered_ident.clone();
+                        let replace_filtered_ident = get_filtered_ident.clone();
                         let set_filtered_ident = get_filtered_ident.clone();
+
                         get_quotes.push(quote! {
                             #(
                                 stringify!(#get_filtered_ident) => {
@@ -115,6 +71,28 @@ pub fn derive(input: TokenStream) -> TokenStream {
                                 }
                             ),*
                         });
+                        get_mut_quotes.push(quote! {
+                            #(
+                                stringify!(#get_mut_filtered_ident) => {
+                                    Ok(&mut self.#get_mut_filtered_ident)
+                                }
+                            ),*
+                        });
+                        take_quotes.push(quote! {
+                            #(
+                                stringify!(#take_filtered_ident) => {
+                                    Ok(mem::take(&mut self.#take_filtered_ident))
+                                }
+                            ),*
+                        });
+                        replace_quotes.push(quote! {
+                            #(
+                                stringify!(#replace_filtered_ident) => {
+                                    Ok(mem::replace(&mut self.#replace_filtered_ident, src))
+                                }
+                            ),*
+                        });
+
                         set_quotes.push(quote! {
                             #(
                                 stringify!(#set_filtered_ident) => {
@@ -126,6 +104,8 @@ pub fn derive(input: TokenStream) -> TokenStream {
                 }
                 quote! {
 
+                    use std::mem;
+
                     #[derive(Debug, Clone)]
                     struct #structinfo {
                         field_names: Vec<String>,
@@ -141,8 +121,12 @@ pub fn derive(input: TokenStream) -> TokenStream {
 
                     trait GetterSetter<T> {
                         fn get(&self, field_string: &String) -> Result<&T, String>;
+                        fn get_mut(&mut self, field_string: &String) -> Result<&mut T, String>;
+                        fn take(&mut self, field_string: &String) -> Result<T, String>;
+                        fn replace(&mut self, field_string: &String, src: T) -> Result<T, String>;
                         fn set(&mut self, field_string: &String, value: T) -> Result<(), String>;
                     }
+
                     #(
                         impl GetterSetter<#set_tys> for #ident {
                             fn get(&self, field_string: &String) -> Result<&#get_tys, String> {
@@ -151,6 +135,25 @@ pub fn derive(input: TokenStream) -> TokenStream {
                                     _ => Err(format!("invalid field name to get '{}'", field_string)),
                                 }
                             }
+                            fn get_mut(&mut self, field_string: &String) -> Result<&mut #get_tys, String> {
+                                match &**field_string {
+                                    #get_mut_quotes,
+                                    _ => Err(format!("invalid field name to get_mut '{}'", field_string)),
+                                }
+                            }
+                            fn take(&mut self, field_string: &String) -> Result<#take_tys, String> {
+                                match &**field_string {
+                                    #take_quotes,
+                                    _ => Err(format!("invalid field name to take '{}'", field_string)),
+                                }
+                            }
+                            fn replace(&mut self, field_string: &String, src: #replace_tys) -> Result<#replace_tys, String> {
+                                match &**field_string {
+                                    #replace_quotes,
+                                    _ => Err(format!("invalid field name to replace '{}'", field_string)),
+                                }
+                            }
+
                             fn set(&mut self, field_string: &String, value: #set_tys) -> Result<(), String>{
                                 match &**field_string {
                                     #set_quotes,
@@ -159,7 +162,21 @@ pub fn derive(input: TokenStream) -> TokenStream {
                             }
                         }
                     )*
+
                     impl #ident {
+
+                        fn swap(&mut self, field_string: &String, field_string_y: &String) -> Result<(), String> {
+                            match (&**field_string, &**field_string_y) {
+                                #(
+                                    (stringify!(#swap_ident), stringify!(#swap_ident2)) => {
+                                        mem::swap::<#swap_tys>(&mut self.#swap_ident, &mut self.#swap_ident2);
+                                        Ok(())
+                                    }
+                                ),*
+                                _ => Err(format!("invalid field name to swap")),
+                            }
+                        }
+
                         fn getenum(&self, field_string: &String) -> Result<#enumname, String> {
                             match &**field_string {
                                 #(stringify!(#idents_getenum) => {
